%   By Feng Cai, Double Resampling
%   Jul.23,2008
%   A Test aiming to compare sSVM, mSVM, SVM+ and svm+MTL
%   A test on breastcancer dataset from uci

clear all;

[age, men, inv, nod, deg, bre, qua, irr,  cla, rec]= textread('breastcancer.dat', ...
    '%f%f%f%f%f%f%f%f%f%f','delimiter',',');

data = [age, men, inv, nod, deg, bre, qua, irr, rec,  cla];

% # of rows
N1 = size(data, 1);

% # of columns
N2 = size(data,2);

alldata = [];
for i = 1:N1
    if size(find(data(i,:)==10000),2)<1
        alldata = [alldata; data(i,:)];
    end
end

% # of rows
N1 = size(alldata, 1);

% # of columns
N2 = size(alldata,2);

% order all data by increasing age
alldata = sortrows(alldata);

alldatag = [alldata(:,2:N2-1) alldata(:,1) alldata(:,N2)];
alldata = alldatag;


% scaled input variables to zero mean 1 variance
for i = 1:N2-1
    tmp1 = min(alldata(:,i));
    tmp2 = max(alldata(:,i));
    alldata(:,i) = (alldata(:,i)-tmp1)/(tmp2-tmp1);
end

train = [];
test = [];
finalaccuracy = [];
finalaccuracy_m = [];
finalaccuracy_plus = [];
finalaccuracy_mtl = [];

% variables to record optimal parameter for each partition
C_sSVM = zeros(5,1);
C_mSVM_1 = zeros(5,1);
C_mSVM_2 = zeros(5,1);
C_mSVM_3 = zeros(5,1);
C_SVMplus = zeros(5,1);
gama_SVMplus = zeros(5,1);
sigma_SVMplus = zeros(5,1);
C_SVMmtl = zeros(5,1);
gama_SVMmtl = zeros(5,1);
sigma_SVMmtl = zeros(5,1);

% debug file
%fid2 = fopen('debug.csv','wt');

for i = 1:5
    testindex = [];
    trainindex = [];
    % compute the index
    for j = 1:N1
        if mod(j,5)==mod(i,5)
            testindex = [testindex j];
        else
            trainindex = [trainindex j];
        end
    end
    
    % train data for this partition
    train = alldata(trainindex,:);
    
    % test data for this partition
    test = alldata(testindex,:);
    
    % construct the test data
    testdata{1}.examples = [];
    testdata{1}.labels = [];
    testdata{2}.examples = [];
    testdata{2}.labels = [];
    testdata{3}.examples = [];
    testdata{3}.labels = [];
    traindata{1}.examples = [];
    traindata{1}.labels = [];
    traindata{2}.examples = [];
    traindata{2}.labels = [];
    traindata{3}.examples = [];
    traindata{3}.labels = [];
    
    % format the test data to use the interface your provided
    % testdata will be the test data for this partition
    for j = 1:size(test,1)
        if test(j,N2-1)<0.4
            testdata{2}.examples = [testdata{2}.examples; test(j,1:N2-1)];
            if test(j,N2)==1
                testdata{2}.labels = [testdata{2}.labels; 1];
            else
                testdata{2}.labels = [testdata{2}.labels; -1];
            end
        elseif test(j,N2-1)<0.57
            testdata{1}.examples = [testdata{1}.examples; test(j,1:N2-1)];
            if test(j,N2)==1
                testdata{1}.labels = [testdata{1}.labels; 1];
            else
                testdata{1}.labels = [testdata{1}.labels; -1];
            end
        else
            testdata{3}.examples = [testdata{3}.examples; test(j,1:N2-1)];
            if test(j,N2)==1
                testdata{3}.labels = [testdata{3}.labels; 1];
            else
                testdata{3}.labels = [testdata{3}.labels; -1];
            end
        end
        
    end
    
    % format the traindata to use the interface you provided
    % traindata will be training data for this partition
    for j = 1:size(train,1)
        if train(j,N2-1)<0.4
            traindata{2}.examples = [traindata{2}.examples; train(j,1:N2-1)];
            if train(j,N2)==1
                traindata{2}.labels = [traindata{2}.labels; 1];
            else
                traindata{2}.labels = [traindata{2}.labels; -1];
            end          
        elseif train(j,N2-1)<0.57
            traindata{1}.examples = [traindata{1}.examples; train(j,1:N2-1)];
            if train(j,N2)==1
                traindata{1}.labels = [traindata{1}.labels; 1];
            else
                traindata{1}.labels = [traindata{1}.labels; -1];
            end
        else
            traindata{3}.examples = [traindata{3}.examples; train(j,1:N2-1)];
            if train(j,N2)==1
                traindata{3}.labels = [traindata{3}.labels; 1];
            else
                traindata{3}.labels = [traindata{3}.labels; -1];
            end
        end
        
    end
    
    % resampling to tune the model parameter
    C = [0.1 1 10 100];
    gama = [10 1 0.1 0.01 0.001];
    sigma = [0.25 0.5 1 2 4];
    N3 = size(train,1);
    
    % double resampling
    for j = 1:5
        testindex = [];
        trainindex = [];
        
        for k = 1:N3
            if mod(k,5)==mod(j,5)
                testindex = [testindex k];
            else
                trainindex = [trainindex k];
            end
        end
        
        % partition on the training data from outer loop
        traintmp = train(trainindex,:);
        testtmp = train(testindex,:);
        
        traintune{1}.examples = [];
        traintune{1}.labels = [];
        traintune{2}.examples = [];
        traintune{2}.labels = [];
        traintune{3}.examples = [];
        traintune{3}.labels = [];
        testtune{1}.examples = [];
        testtune{1}.labels = [];
        testtune{2}.examples = [];
        testtune{2}.labels = [];
        testtune{3}.examples = [];
        testtune{3}.labels = [];
        
        for k = 1:size(traintmp,1)
            if traintmp(k,N2-1)<0.4
                traintune{2}.examples = [traintune{2}.examples; traintmp(k,1:N2-1)];
                if traintmp(k,N2)==1
                    traintune{2}.labels = [traintune{2}.labels; 1];
                else
                    traintune{2}.labels = [traintune{2}.labels; -1];
                end
            elseif traintmp(k,N2-1)<0.57
                traintune{1}.examples = [traintune{1}.examples; traintmp(k,1:N2-1)];
                if traintmp(k,N2)==1
                    traintune{1}.labels = [traintune{1}.labels; 1];
                else
                    traintune{1}.labels = [traintune{1}.labels; -1];
                end
            else
                traintune{3}.examples = [traintune{3}.examples; traintmp(k,1:N2-1)];
                if traintmp(k,N2)==1
                    traintune{3}.labels = [traintune{3}.labels; 1];
                else
                    traintune{3}.labels = [traintune{3}.labels; -1];
                end
            end
                        
        end
        
        for k = 1:size(testtmp,1)
            if testtmp(k,N2-1)<0.4
                testtune{2}.examples = [testtune{2}.examples; testtmp(k,1:N2-1)];
                if testtmp(k,N2)==1
                    testtune{2}.labels = [testtune{2}.labels; 1];
                else
                    testtune{2}.labels = [testtune{2}.labels; -1];
                end
            elseif testtmp(k,N2-1)<0.57
                testtune{1}.examples = [testtune{1}.examples; testtmp(k,1:N2-1)];
                if testtmp(k,N2)==1
                    testtune{1}.labels = [testtune{1}.labels; 1];
                else
                    testtune{1}.labels = [testtune{1}.labels; -1];
                end
            else
                testtune{3}.examples = [testtune{3}.examples; testtmp(k,1:N2-1)];
                if testtmp(k,N2)==1
                    testtune{3}.labels = [testtune{3}.labels; 1];
                else
                    testtune{3}.labels = [testtune{3}.labels; -1];
                end
            end
                        
        end
        
        % compute accuracy for different model parameters
        for inx1 = 1:size(C,2)
                % setting paramters
            para.C = C(inx1);
            para.g = 1;
            para.kernel1 = 'jcb';   % decision space kernel
            para.kerneloption1 =1;  % decision space kernel parameter
            para.kernel2 ='gaussian'; % correcting space kernel
            para.kerneloption2 = 1;   % correcting space kernle parameter
            
            svm_classifier = trainSVMplusMTL(traintune,'svm',para);
            accuracy(i,j,inx1) = predSVM(svm_classifier,testtune);
            
            svm_classifier1 = trainSVMplusMTL(traintune(1),'svm',para);
            accuracy_m1(i,j,inx1) = predSVM(svm_classifier1,testtune(1));
            
            svm_classifier2 = trainSVMplusMTL(traintune(2),'svm',para);
            accuracy_m2(i,j,inx1) = predSVM(svm_classifier2,testtune(2));
            
            svm_classifier3 = trainSVMplusMTL(traintune(3),'svm',para);
            accuracy_m3(i,j,inx1) = predSVM(svm_classifier3,testtune(3));
            
            
            for inx2 = 1:size(gama,2)
                for inx3 = 1:size(sigma,2)
                    para.g = gama(inx2);
                    para.kerneloption2 = sigma(inx3);
                    
                    svmp_classifier = trainSVMplusMTL(traintune,'svmplus',para);
                    accuracy_plus(i,j,inx1,inx2,inx3) = predSVMplus(svmp_classifier,testtune);
       
                    
                    svmmtl_classifier = trainSVMplusMTL(traintune,'svmmtl',para);
                    accuracy_mtl(i,j,inx1,inx2,inx3) = predSVMMTL(svmmtl_classifier,testtune);
                end
            end
        end
    end
    
    % select optimal parameters
    innersvm_maxaccuracy(i) = 0;
    opt_C(i) = 0.001;
    innersvmm1_maxaccuracy(i) = 0;
    opt_Cm1(i) = 0.001;
    innersvmm2_maxaccuracy(i) = 0;
    opt_Cm2(i) = 0.001;
    innersvmm3_maxaccuracy(i) = 0;
    opt_Cm3(i) = 0.001;
    innersvmplus_maxaccuracy(i) = 0;
    opt_Cplus(i) = 0.1;
    opt_gamaplus(i) = 10;
    opt_sigmaplus(i) = 0.5;
    innersvmmtl_maxaccuracy(i) = 0;
    opt_Cmtl(i) = 0.1;
    opt_gamamtl(i) = 10;
    opt_sigmamtl(i) = 0.5;
   
    for inx1 = 1:size(C,2)
        acc_svm = mean(accuracy(i,:,inx1));
        if innersvm_maxaccuracy(i)<acc_svm
            innersvm_maxaccuracy(i)=acc_svm;
            opt_C(i) = C(inx1);
        end
        
        acc_svm1 = mean(accuracy_m1(i,:,inx1));
        if innersvmm1_maxaccuracy(i)<acc_svm1
            innersvmm1_maxaccuracy(i) = acc_svm1;
            opt_Cm1(i) = C(inx1);
        end
        
        acc_svm2 = mean(accuracy_m2(i,:,inx1));
        if innersvmm2_maxaccuracy(i)<acc_svm2
            innersvmm2_maxaccuracy(i) = acc_svm2;
            opt_Cm2(i) = C(inx1);
        end
        
        acc_svm3 = mean(accuracy_m3(i,:,inx1));
        if innersvmm3_maxaccuracy(i)<acc_svm3
            innersvmm3_maxaccuracy(i) = acc_svm3;
            opt_Cm3(i) = C(inx1);
        end
                
        for inx2 = 1:size(gama,2)
            for inx3 = 1:size(sigma,2)
                acc_svmplus = mean(accuracy_plus(i,:,inx1,inx2,inx3));
                if innersvmplus_maxaccuracy(i)<acc_svmplus
                    innersvmplus_maxaccuracy(i)=acc_svmplus;
                    opt_Cplus(i) = C(inx1);
                    opt_gamaplus(i) = gama(inx2);
                    opt_sigmaplus(i) = sigma(inx3);
                end
                
                acc_svmmtl = mean(accuracy_mtl(i,:,inx1,inx2,inx3));
                if innersvmmtl_maxaccuracy(i)<acc_svmmtl
                    innersvmmtl_maxaccuracy(i)=acc_svmmtl;
                    opt_Cmtl(i) = C(inx1);
                    opt_gamamtl(i) = gama(inx2);
                    opt_sigmamtl(i) = sigma(inx3);
                end
            end
        end
    end
    
    para.g = 1;
    para.kernel1 = 'jcb';   % decision space kernel
    para.kerneloption1 =1;  % decision space kernel parameter
    para.kernel2 ='gaussian'; % correcting space kernel
    para.kerneloption2 = 1;   % correcting space kernle parameter
            
    % compute the prediction error
    para.C = opt_C(i);
    C_sSVM(i,1) = opt_C(i);
    svm_classifier = trainSVMplusMTL(traindata,'svm',para);
    svm_pred = predSVM(svm_classifier,testdata);
    finalaccuracy = [finalaccuracy; svm_pred];
    
    para.C = opt_Cm1(i);
    C_mSVM_1(i,1) = opt_Cm1(i);
    svm_classifier1 = trainSVMplusMTL(traindata(1),'svm',para);
    svm_pred1 = predSVM(svm_classifier1,testdata(1));
    
    para.C = opt_Cm2(i);
    C_mSVM_2(i,1) = opt_Cm2(i);
    svm_classifier2 = trainSVMplusMTL(traindata(2),'svm',para);
    svm_pred2 = predSVM(svm_classifier2,testdata(2));
    
    para.C = opt_Cm3(i);
    C_mSVM_3(i,1) = opt_Cm3(i);
    svm_classifier3 = trainSVMplusMTL(traindata(3),'svm',para);
    svm_pred3 = predSVM(svm_classifier3,testdata(3));
    
    
    finalaccuracy_m = [finalaccuracy_m; (svm_pred1*size(testdata{1}.examples,1)...
        +svm_pred2*size(testdata{2}.examples,1)+svm_pred3*size(testdata{3}.examples,1))/(size(testdata{1}.examples,1)+...
        size(testdata{2}.examples,1)+size(testdata{3}.examples,1))];
    
    
    para.C = opt_Cplus(i);
    C_SVMplus(i,1) = opt_Cplus(i);
    para.g = opt_gamaplus(i);
    gama_SVMplus(i,1) = opt_gamaplus(i);
    para.kerneloption2 = opt_sigmaplus(i);
    sigma_SVMplus(i,1) = opt_sigmaplus(i);
    svmp_classifier = trainSVMplusMTL(traindata,'svmplus',para);
    svmp_pred = predSVMplus(svmp_classifier,testdata);
    finalaccuracy_plus = [finalaccuracy_plus; svmp_pred];
    
    para.C = opt_Cmtl(i);
    C_SVMmtl(i,1) = opt_Cmtl(i);
    para.g = opt_gamamtl(i);
    gama_SVMmtl(i,1) = opt_gamamtl(i);
    para.kerneloption2 = opt_sigmamtl(i);
    sigma_SVMmtl(i,1) = opt_sigmamtl(i);
    svmmtl_classifier = trainSVMplusMTL(traindata,'svmmtl',para);
    svmmtl_pred = predSVMMTL(svmmtl_classifier,testdata);
    finalaccuracy_mtl = [finalaccuracy_mtl; svmmtl_pred];
end

fprintf('The prediction accuracy results:\n');
fprintf('sSVM:    %f ', mean(finalaccuracy));
fprintf([setstr(177), ' %f\n'], std(finalaccuracy));
fprintf('SVM+:    %f ', mean(finalaccuracy_plus));
fprintf([setstr(177), ' %f\n'], std(finalaccuracy_plus));
fprintf('mSVM:    %f ', mean(finalaccuracy_m));
fprintf([setstr(177), ' %f\n'], std(finalaccuracy_m));
fprintf('SVM+MTL: %f ', mean(finalaccuracy_mtl));
fprintf([setstr(177), ' %f\n'], std(finalaccuracy_mtl));